import 'package:xpx_chain_sdk/xpx_sdk.dart';

void main() async {
  const baseUrl = 'http://bctestnet2.brimstone.xpxsirius.io:3000';

  const networkType = publicTest;

  /// Creating a client instance
  final client = SiriusClient.fromUrl(baseUrl, null);

  /// Create an Address from a given public key.
  final accountOne = PublicAccount.fromPublicKey(
      'E94CF58CEFCB1E9F590B8D190807CA4B6EF51A39847A64CDC6DB344B5AE16A6E',
      networkType);

  /// Create an Address from a given public key.
  final accountTwo = PublicAccount.fromPublicKey(
      'F6A9951DDB2B9EF2C7EB5132654562FC6D2308B42AEAAFD11097B232F429AF70',
      networkType);

  // Returns the account information.
  try {
    final result = await client.account.getAccountInfo(accountOne.address);
    print(result);
  } on Exception catch (e) {
    print('Exception when calling Account->GetAccountInfo: $e\n');
  }

  // Returns the account information for an array of accounts.
  try {
    final result = await client.account
        .getAccountsInfo([accountOne.address, accountTwo.address]);
    print(result);
  } on Exception catch (e) {
    print('Exception when calling Account->GetAccountsInfo: $e\n');
  }

  // Returns the [multisig account] information.
  try {
    final result = await client.account.getAccountMultisig(accountOne.address);
    print(result);
  } on Exception catch (e) {
    print('Exception when calling Account->GetAccountMultisig: $e\n');
  }

  // Returns the [multisig account] graph.
  try {
    final result =
        await client.account.getAccountMultisigGraph(accountOne.address);
    print(result);
  } on Exception catch (e) {
    print('Exception when calling Account->GetAccountMultisigGraph: $e\n');
  }

  // Gets an array of transactions for which an account is the sender or receiver.
  try {
    final result = await client.account.transactions(accountOne);
    print(result);
  } on Exception catch (e) {
    print('Exception when calling Account->Transactions: $e\n');
  }

  // Gets an array of incoming transactions.
  // A transaction is said to be incoming with respect to an account if the account is the recipient of the transaction.
  try {
    final result = await client.account.incomingTransactions(accountOne);
    print(result);
  } on Exception catch (e) {
    print('Exception when calling Account->IncomingTransactions: $e\n');
  }

  // Gets an array of outgoing transactions.
  // A transaction is said to be outgoing with respect to an account if the account is the sender of the transaction.
  try {
    final result = await client.account.outgoingTransactions(accountOne);
    print(result);
  } on Exception catch (e) {
    print('Exception when calling Account->OutgoingTransactions: $e\n');
  }

  // Gets the array of transactions not included in a block where an account is the sender or receiver.
  try {
    final result = await client.account.unconfirmedTransactions(accountOne);
    print(result);
  } on Exception catch (e) {
    print('Exception when calling Account->UnconfirmedTransactions: $e\n');
  }

  // Gets an array of [aggregate bonded transactions] where the account is the sender or requires to cosign the transaction.
  try {
    final result = await client.account.partialTransactions(accountOne);
    print(result);
  } on Exception catch (e) {
    print('Exception when calling Account->AggregateBondedTransactions: $e\n');
  }

  // Returns friendly names for accounts.
  try {
    final result = await client.account
        .getAccountsNames([accountOne.address, accountTwo.address]);
    print(result);
  } on Exception catch (e) {
    print('Exception when calling Account->GetAccountsNames: $e\n');
  }

  // Returns the [configurable properties] for a given account.
  try {
    final result =
        await client.account.getAccountProperties(accountOne.address);
    print(result);
  } on Exception catch (e) {
    print('Exception when calling Account->GetAccountProperties: $e\n');
  }

  // Returns the [configurable properties] for a given array of addresses.
  try {
    final result = await client.account
        .getAccountsProperties([accountOne.address, accountTwo.address]);
    print(result);
  } on Exception catch (e) {
    print('Exception when calling Account->GetAccountsProperties: $e\n');
  }
}
