part of xpx_chain_sdk.transaction;

/// A deposit before announcing aggregate bonded transactions.
/// Alias: LockFundsTransaction
///
class LockFundsTransaction extends AbstractTransaction implements Transaction {
  LockFundsTransaction(Deadline deadline, Mosaic mosaic, Uint64 duration,
      SignedTransaction signedTx, int networkType)
      : super() {
    if (signedTx.transactionType != TransactionType.aggregateBonded.value) {
      throw errEmptyModifications;
    } else {
      version = lockVersion;
      this.deadline = deadline;
      type = TransactionType.lock;
      this.networkType = networkType;
      this.mosaic = mosaic;
      this.duration = duration;
      signedTransaction = signedTx;
    }
  }

  LockFundsTransaction.fromDTO(LockFundsTransactionInfoDTO dto)
      : super.fromDto(dto._transaction!, dto.meta!) {
    mosaic = Mosaic(MosaicId.fromUint64(dto._transaction!._assetId!.toUint64()),
        dto._transaction!._amount!.toUint64());
    duration = dto._transaction!._duration!.toUint64();
    signedTransaction = SignedTransaction(0x4148, '', dto._transaction!._hash);
  }

  Mosaic? mosaic;
  Uint64? duration;
  SignedTransaction? signedTransaction;

  int get size => _size();

  @override
  TransactionType? entityType() => type;

  AbstractTransaction get abstractTransaction => absTransaction();

  @override
  String toString() => '{\n'
      '\t"abstractTransaction": ${_absToString()}\n'
      '\t"mosaic": $mosaic,\n'
      '\t"duration": $duration,\n'
      '\t"signedTxHash": ${signedTransaction!.hash},\n'
      '}\n';

  @override
  Map<String, dynamic> toJson() {
    final data = <String, dynamic>{};
    data['abstractTransaction'] = _absToJson();
    data['mosaic'] = mosaic;
    data['duration'] = duration;
    data['signedTransaction'] = signedTransaction;

    return data;
  }

  @override
  int _size() => lockSize;

  @override
  AbstractTransaction absTransaction() => _absTransaction();

  @override
  Uint8List generateBytes() {
    final builder = fb.Builder(initialSize: 0);

    final mV = builder.writeListUint32(mosaic!.assetId!.toIntArray());

    final maV = builder.writeListUint32(mosaic!.amount!.toIntArray());

    final dV = builder.writeListUint32(duration!.toIntArray());

    final List<int> h = hex.decode(signedTransaction!.hash!);

    final hV = builder.writeListUint8(h);

    final vectors = _generateVector(builder);

    final txnBuilder = $buffer.LockFundsTransactionBufferBuilder(builder)
      ..begin()
      ..addSize(_size())
      ..addMosaicIdOffset(mV)
      ..addMosaicAmountOffset(maV)
      ..addDurationOffset(dV)
      ..addHashOffset(hV);
    _buildVector(builder, vectors);

    final codedLockFunds = txnBuilder.finish();
    builder.finish(codedLockFunds);
    return lockFundsTransactionSchema().serialize(builder.buffer);
  }
}
