part of xpx_chain_sdk.transaction;

/// Register a namespace to organize your assets.
/// Announce a [RegisterNamespaceTransaction] to register and re-rent a namespace.
///
class RegisterNamespaceTransaction extends AbstractTransaction
    implements Transaction {
  RegisterNamespaceTransaction.createRoot(Deadline deadline,
      String rootNamespaceName, Uint64 duration, int networkType)
      : super() {
    if (rootNamespaceName.isEmpty) {
      throw errInvalidNamespaceName;
    } else if (duration == null) {
      throw errNullDuration;
    } else {
      version = registerNamespaceVersion;
      this.deadline = deadline;
      type = TransactionType.registerNamespace;
      namespaceId = NamespaceId.fromName(rootNamespaceName);
      this.networkType = networkType;
      namespaceName = rootNamespaceName;
      namespaceType = NamespaceType.root;
      this.duration = duration;
    }
  }

  RegisterNamespaceTransaction.createSub(Deadline deadline,
      String subNamespaceName, String rootNamespaceName, int networkType)
      : super() {
    if (subNamespaceName.isEmpty) {
      throw errInvalidNamespaceName;
    } else if (rootNamespaceName.isEmpty) {
      throw errInvalidNamespaceName;
    } else {
      version = registerNamespaceVersion;
      this.deadline = deadline;
      type = TransactionType.registerNamespace;
      parentId = NamespaceId.fromName(rootNamespaceName);
      namespaceId = NamespaceId(
          id: generateNamespaceId(subNamespaceName, parentId!.toUint64()!));
      this.networkType = networkType;
      namespaceName = subNamespaceName;
      namespaceType = NamespaceType.sub;
    }
  }

  RegisterNamespaceTransaction.fromDTO(RegisterNamespaceTransactionInfoDTO dto)
      : super.fromDto(dto.transaction!, dto.meta!) {
    namespaceId = NamespaceId(id: dto.transaction!.namespaceId!.toUint64());
    namespaceType = dto.transaction!.namespaceType == 0
        ? NamespaceType.root
        : NamespaceType.sub;
    namespaceName = dto.transaction!.name;
    if (namespaceType == NamespaceType.root) {
      duration = dto.transaction!.duration!.toUint64();
    } else {
      parentId = NamespaceId(id: dto.transaction!.parentId!.toUint64());
    }
  }

  late NamespaceId namespaceId;
  NamespaceType? namespaceType;
  String? namespaceName;
  Uint64? duration;
  NamespaceId? parentId;

  int get size => _size();

  @override
  TransactionType? entityType() => type;

  AbstractTransaction get abstractTransaction => absTransaction();

  static List<RegisterNamespaceTransaction> listFromDTO(
          List<RegisterNamespaceTransactionInfoDTO> data) =>
      data.isEmpty
          ? <RegisterNamespaceTransaction>[]
          : data
              .map((value) => RegisterNamespaceTransaction.fromDTO(value))
              .toList();

  @override
  String toString() => encoder.convert(this);

  @override
  Map<String, dynamic> toJson() {
    final Map<String, dynamic> val = {}..addAll(_absToJson());

    void writeNotNull(String key, value) {
      if (value != null) {
        val[key] = value;
      }
    }

    writeNotNull('namespaceId', namespaceId.toHex());
    writeNotNull('namespaceType', namespaceType.toString());
    writeNotNull('namespaceName', namespaceName);
    writeNotNull('parentId', parentId);
    writeNotNull('duration', duration);
    return val;
  }

  @override
  AbstractTransaction absTransaction() => _absTransaction();

  @override
  Uint8List generateBytes() {
    final builder = fb.Builder(initialSize: 0);

    final nV = builder.writeListUint32(namespaceId.toIntArray());
    int dV;
    if (namespaceType == NamespaceType.root) {
      dV = builder.writeListUint32(duration!.toIntArray());
    } else {
      dV = builder.writeListUint32(parentId!.toIntArray());
    }

    final n = builder.writeString(namespaceName!);

    final vector = _generateVector(builder);

    final txnBuilder =
        $buffer.RegisterNamespaceTransactionBufferBuilder(builder)
          ..begin()
          ..addSize(_size())
          ..addNamespaceType(namespaceType!.index)
          ..addDurationParentIdOffset(dV)
          ..addNamespaceIdOffset(nV)
          ..addNamespaceNameSize(namespaceName!.length)
          ..addNamespaceNameOffset(n);
    _buildVector(builder, vector);

    final codedRegisterNamespace = txnBuilder.finish();
    builder.finish(codedRegisterNamespace);
    return registerNamespaceTransactionSchema().serialize(builder.buffer);
  }

  @override
  int _size() => registerNamespaceHeaderSize + namespaceName!.length;
}
