part of xpx_chain_sdk.transaction;

/// Send mosaics and messages between two accounts.
/// Announce a [TransferTransaction] to send [Mosaic] or [Message] between two [Account].
///
class TransferTransaction extends AbstractTransaction implements Transaction {
  TransferTransaction(Deadline deadline, Address this.recipient,
      List<Mosaic> this.mosaics, Message this.message, int? networkType)
      : super() {
    version = transferVersion;
    this.deadline = deadline;
    type = TransactionType.transfer;
    this.networkType = networkType;
  }

  TransferTransaction.fromDTO(TransferTransactionInfoDTO dto)
      : super.fromDto(dto._transaction!, dto.meta!) {
    mosaics = Mosaic.listFromDTO(dto._transaction!._mosaics);
    recipient = Address.fromEncoded(dto._transaction!._recipient!);
    message = dto._transaction!._message != null
        ? Message.fromDTO(dto._transaction!._message!)
        : null;
  }

  List<Mosaic>? mosaics;
  Address? recipient;
  Message? message;

  static List<TransferTransaction> listFromDTO(
          List<TransferTransactionInfoDTO> json) =>
      json.isEmpty
          ? []
          : json.map((value) => TransferTransaction.fromDTO(value)).toList();

  @override
  String toString() => encoder.convert(this);

  @override
  Map<String, dynamic> toJson() {
    final Map<String, dynamic> val = {}..addAll(_absToJson());

    void writeNotNull(String key, value) {
      if (value != null) {
        val[key] = value;
      }
    }

    writeNotNull('mosaics', mosaics);
    writeNotNull('recipient', recipient);
    writeNotNull('message', message);

    return val;
  }

  int messageSize() => message!.payload.length + 1;

  @override
  int _size() =>
      transferHeaderSize +
      (mosaicIdSize + amountSize) * mosaics!.length +
      messageSize();

  @override
  AbstractTransaction absTransaction() => _absTransaction();

  @override
  Uint8List generateBytes() {
    final builder = fb.Builder(initialSize: 0);

    /// Create mosaics
    final mb = List.generate(mosaics!.length, (_) => 0);
    int i = 0;
    for (final mosaic in mosaics!) {
      final id = builder.writeListUint32(mosaic.assetId!.toIntArray());
      final amount = builder.writeListUint32(mosaic.amount!.toIntArray());

      final ms = $buffer.MosaicBufferBuilder(builder)
        ..begin()
        ..addIdOffset(id)
        ..addAmountOffset(amount);
      mb[i] = ms.finish();
      i++;
    }

    /// Create message;
    final mp = builder.writeListUint8(this.message!.payload);
    final message = $buffer.MessageBufferBuilder(builder)
      ..begin()
      ..addType(this.message!.type.value)
      ..addPayloadOffset(mp);
    final int m = message.finish();

    final recipient = base32.decode(this.recipient!.address);

    final rV = builder.writeListUint8(recipient);

    final mV = builder.writeList(mb);

    final vectors = _generateVector(builder);

    final txnBuilder = $buffer.TransferTransactionBufferBuilder(builder)
      ..begin()
      ..addSize(_size())
      ..addRecipientOffset(rV)
      ..addNumMosaics(mosaics!.length)
      ..addMessageSize(this.message!.payload.length + 1)
      ..addMessageOffset(m)
      ..addMosaicsOffset(mV);
    _buildVector(builder, vectors);

    final codedTransfer = txnBuilder.finish();

    builder.finish(codedTransfer);

    return transferTransactionSchema().serialize(builder.buffer);
  }

  @override
  TransactionType? entityType() => type;
}
